# Reference

<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

### Classes

#### Public Classes

* [`pxe_install`](#pxe_install): PXE Install server

#### Private Classes

* `pxe_install::apache`: Configure webserver for install server
* `pxe_install::dhcp`: Setup DHCP server
* `pxe_install::params`: Default parameters
* `pxe_install::syslinux`: Download syslinux archive
* `pxe_install::tftp`: Setup ftp server
* `pxe_install::winipxe`: Install windows ipxe files

### Defined types

#### Public Defined types

* [`pxe_install::parent_dirs`](#pxe_install--parent_dirs): Create directories recursivly

#### Private Defined types

* `pxe_install::kickstart`: Create kickstart file
* `pxe_install::partitioning::debian`: Create a partion table for Debian systems
* `pxe_install::partitioning::redhat`: ACreate partitioning for redhat/CentOPS systems
* `pxe_install::partitioning::ubuntu`: Create a partion table for Ubuntu systems
* `pxe_install::samba::host`: Create a host entry for a Windows node
* `pxe_install::samba::unattend`: create unattended boot file
* `pxe_install::tftp::host`: Create a tftp server entry for a host.

### Functions

* [`get_partition_devices`](#get_partition_devices): get_partition_devices.rb Examin partitioning information of Debian/Ubuntu hosts and get all used disk devices
* [`pxe_install::hex_ip`](#pxe_install--hex_ip): Convert a ip from digits into hex.

### Tasks

* [`create_password`](#create_password): Outputs a SHA512 encrypted password for use with node installation.
* [`maintain_netinstaller`](#maintain_netinstaller): Maintain CentOS/Ubuntu/Debian net installer files

## Classes

### <a name="pxe_install"></a>`pxe_install`

Set up an PXE install server with dhcp, tftp and webserver for serving files

#### Examples

##### 

```puppet
include pxe_install
```

#### Parameters

The following parameters are available in the `pxe_install` class:

* [`installserverip`](#-pxe_install--installserverip)
* [`installserver`](#-pxe_install--installserver)
* [`repo_server`](#-pxe_install--repo_server)
* [`repo_server_ip`](#-pxe_install--repo_server_ip)
* [`repos_dir`](#-pxe_install--repos_dir)
* [`repos_url`](#-pxe_install--repos_url)
* [`scriptdir`](#-pxe_install--scriptdir)
* [`scripturl`](#-pxe_install--scripturl)
* [`kickstart_dir`](#-pxe_install--kickstart_dir)
* [`kickstart_url`](#-pxe_install--kickstart_url)
* [`puppetmaster`](#-pxe_install--puppetmaster)
* [`puppetmasterip`](#-pxe_install--puppetmasterip)
* [`services`](#-pxe_install--services)
* [`machines`](#-pxe_install--machines)
* [`status_allow_from`](#-pxe_install--status_allow_from)
* [`enabled`](#-pxe_install--enabled)
* [`ssl_cert`](#-pxe_install--ssl_cert)
* [`ssl_key`](#-pxe_install--ssl_key)
* [`ssl_chain`](#-pxe_install--ssl_chain)
* [`ssl_certs_dir`](#-pxe_install--ssl_certs_dir)
* [`documentroot`](#-pxe_install--documentroot)
* [`create_aliases`](#-pxe_install--create_aliases)
* [`challenge_password`](#-pxe_install--challenge_password)
* [`add_hosts_entries`](#-pxe_install--add_hosts_entries)
* [`install_curl`](#-pxe_install--install_curl)
* [`install_unzip`](#-pxe_install--install_unzip)
* [`syslinux_url`](#-pxe_install--syslinux_url)
* [`syslinux_name`](#-pxe_install--syslinux_name)
* [`syslinux_version`](#-pxe_install--syslinux_version)
* [`ipxefile`](#-pxe_install--ipxefile)
* [`mirrors`](#-pxe_install--mirrors)
* [`defaults`](#-pxe_install--defaults)
* [`purge_apache_configs`](#-pxe_install--purge_apache_configs)

##### <a name="-pxe_install--installserverip"></a>`installserverip`

Data type: `String`

The ip of the install server

##### <a name="-pxe_install--installserver"></a>`installserver`

Data type: `String`

Hostname of the install server

##### <a name="-pxe_install--repo_server"></a>`repo_server`

Data type: `String`

Hostname of the server hosting the package repositories

##### <a name="-pxe_install--repo_server_ip"></a>`repo_server_ip`

Data type: `String`

IP of the server hosting the package repositories

##### <a name="-pxe_install--repos_dir"></a>`repos_dir`

Data type: `Stdlib::Unixpath`

Directory location of the package repositories

##### <a name="-pxe_install--repos_url"></a>`repos_url`

Data type: `String`

URI where the pacjaqges can be found on the repo server

##### <a name="-pxe_install--scriptdir"></a>`scriptdir`

Data type: `Stdlib::Unixpath`

Directory location where to cfreate scripts and prepared files

##### <a name="-pxe_install--scripturl"></a>`scripturl`

Data type: `String`

URL where to find the scripts

##### <a name="-pxe_install--kickstart_dir"></a>`kickstart_dir`

Data type: `Stdlib::Unixpath`

Directory location for the kickstart/preseed files

##### <a name="-pxe_install--kickstart_url"></a>`kickstart_url`

Data type: `String`

Url where to download the kickstart/preseed files

##### <a name="-pxe_install--puppetmaster"></a>`puppetmaster`

Data type: `Optional[String]`

The hostname of the Puppet master server

Default value: `undef`

##### <a name="-pxe_install--puppetmasterip"></a>`puppetmasterip`

Data type: `Optional[String]`

The ip of the Puppet master server

Default value: `undef`

##### <a name="-pxe_install--services"></a>`services`

Data type: `Optional[Hash]`

Hash with services to configure. Valid are dhcp and tftp. Each service is a hash of how to configure
that service.

Default value: `undef`

##### <a name="-pxe_install--machines"></a>`machines`

Data type: `Optional[Hash]`

Hash with all machines to get installed. Each entry i a whole description of a machine with ips,
partitions and so on.

Default value: `undef`

##### <a name="-pxe_install--status_allow_from"></a>`status_allow_from`

Data type: `Array`

Allow Apache status URLs from these ips

Default value: `['127.0.0.1']`

##### <a name="-pxe_install--enabled"></a>`enabled`

Data type: `Boolean`

Installserver is enbaled or not

Default value: `true`

##### <a name="-pxe_install--ssl_cert"></a>`ssl_cert`

Data type: `String`

Location of the SSL certificate for the webserver to use. he file has to exist on the install server.

Default value: `'/etc/pki/httpd/repos.example.com/repos.example.com.cer'`

##### <a name="-pxe_install--ssl_key"></a>`ssl_key`

Data type: `String`

Location of the SSL key for the webserver to use. he file has to exist on the install server.

Default value: `'/etc/pki/httpd/repos.example.com/repos.example.com.key'`

##### <a name="-pxe_install--ssl_chain"></a>`ssl_chain`

Data type: `String`

Location of the SSL certificate chain for the webserver to use. he file has to exist on the install server.

Default value: `'/etc/pki/httpd/repos.example.com/fullchain.cer'`

##### <a name="-pxe_install--ssl_certs_dir"></a>`ssl_certs_dir`

Data type: `String`

Directory where all files for the cdertificates reside.

Default value: `'/etc/pki/httpd/repos.example.com/'`

##### <a name="-pxe_install--documentroot"></a>`documentroot`

Data type: `String`

Document root for the webserver.

Default value: `'/var/www/html'`

##### <a name="-pxe_install--create_aliases"></a>`create_aliases`

Data type: `Boolean`

Create webserver aliases

Default value: `true`

##### <a name="-pxe_install--challenge_password"></a>`challenge_password`

Data type: `Sensitive[String]`

Sensitive challenge password for auto signing cert requests.

##### <a name="-pxe_install--add_hosts_entries"></a>`add_hosts_entries`

Data type: `Boolean`

Add install server and puppet server to /etc/hosts file.

Default value: `false`

##### <a name="-pxe_install--install_curl"></a>`install_curl`

Data type: `Boolean`

Install curl package

Default value: `$pxe_install::params::install_curl`

##### <a name="-pxe_install--install_unzip"></a>`install_unzip`

Data type: `Boolean`

Install unzip package

Default value: `$pxe_install::params::install_unzip`

##### <a name="-pxe_install--syslinux_url"></a>`syslinux_url`

Data type: `Stdlib::HTTPSUrl`

The url where to download the syslinux archive.

Default value: `$pxe_install::params::syslinux_url`

##### <a name="-pxe_install--syslinux_name"></a>`syslinux_name`

Data type: `String`

Ther name of the archive.

Default value: `$pxe_install::params::syslinux_name`

##### <a name="-pxe_install--syslinux_version"></a>`syslinux_version`

Data type: `String`

The version of the archive.

Default value: `$pxe_install::params::syslinux_version`

##### <a name="-pxe_install--ipxefile"></a>`ipxefile`

Data type: `Stdlib::HTTPUrl`

The url to download the ipxe file.

Default value: `$pxe_install::params::ipxefile`

##### <a name="-pxe_install--mirrors"></a>`mirrors`

Data type: `Hash`

Hash with mirror definitions.

Default value: `$pxe_install::params::mirrors`

##### <a name="-pxe_install--defaults"></a>`defaults`

Data type: `Hash`

Default values.

Default value: `$pxe_install::params::defaults`

##### <a name="-pxe_install--purge_apache_configs"></a>`purge_apache_configs`

Data type: `Boolean`

Boolean to indicate that all Apache configurations not maintained by the Apache Puppet module should be deleted.

Default value: `false`

## Defined types

### <a name="pxe_install--parent_dirs"></a>`pxe_install::parent_dirs`

Create all missing directories

 }

#### Examples

##### 

```puppet
pxe_installarent_dirs{ 'create script dir':
 dir_path => '/var/www/scripts',
```

#### Parameters

The following parameters are available in the `pxe_install::parent_dirs` defined type:

* [`dir_path`](#-pxe_install--parent_dirs--dir_path)

##### <a name="-pxe_install--parent_dirs--dir_path"></a>`dir_path`

Data type: `Stdlib::Unixpath`

The directories to be created.

## Functions

### <a name="get_partition_devices"></a>`get_partition_devices`

Type: Ruby 4.x API

get_partition_devices.rb
Examin partitioning information of Debian/Ubuntu hosts and get all
used disk devices

#### `get_partition_devices(Hash $partitioning)`

get_partition_devices.rb
Examin partitioning information of Debian/Ubuntu hosts and get all
used disk devices

Returns: `String`

##### `partitioning`

Data type: `Hash`



### <a name="pxe_install--hex_ip"></a>`pxe_install::hex_ip`

Type: Puppet Language

Convert a ip from digits into hex.

#### `pxe_install::hex_ip(String $ip)`

The pxe_install::hex_ip function.

Returns: `String` IP in hey format

##### `ip`

Data type: `String`

The ip to convert, e. g. 10.0.0.1

## Tasks

### <a name="create_password"></a>`create_password`

Outputs a SHA512 encrypted password for use with node installation.

**Supports noop?** false

#### Parameters

##### `password`

Data type: `String[1]`

The password to encrypt

### <a name="maintain_netinstaller"></a>`maintain_netinstaller`

Maintain CentOS/Ubuntu/Debian net installer files

**Supports noop?** false

#### Parameters

##### `tftp_basedir`

Data type: `String[1]`

The base directory of the tftpd server where the net installer files go to.

##### `archive_url`

Data type: `String[1]`

Where to download he net installer from.

##### `os`

Data type: `Enum[centos,ubuntu,debian,fedora,alma,rocky,redhat]`

The operating system.

##### `os_version`

Data type: `String[1]`

The operting system version, depending on the OS.

##### `os_subversion`

Data type: `Optional[String[1]]`

The operating system subversion for CentOS like 'u9'.

