require 'spec_helper'

describe 'pxe_install::dhcp' do
  let(:pre_condition) do
    <<-EOF
    class { 'pxe_install':
      services => {
        tftp => {
          packages => ['tftp-server', 'xinetd'],
          packages_ensure => 'installed',
          'port' => 69,
          'user' => 'root',
          'group' => 'root',
          'directory' => '/var/lib/tftpboot',
          'pxelinux' => 'pxelinux.cfg',
          'address' => '10.0.0.2',
          'tftpserverbin' => '/usr/sbin/in.tftpd',
          'service' => 'xinetd',
          'service_ensure' => 'running',
          'service_enable' => true,
        },
      },
    }
    EOF
  end

  on_supported_os.each do |os, os_facts|
    context "on #{os}" do
      let(:facts) { os_facts }
      let(:params) do
        {
          'dhcp' => {
            'service_ensure' => 'running',
            'interfaces' => ['ens192'],
            'dns_servers' => ['127.0.0.1'],
            'ntp_servers' => ['127.0.0.1'],
            'next_server' => '127.0.0.1',
            'domain_names' => ['localdomain'],
            'pxefilename' => 'pxelinux.0',
            'ddns_update_style' => 'omapiport',
            'max_lease_time' => 86_400,
            'default_lease_time' => 86_400,
            'logfacility' => 'local7',
            'option_code_50_label' => 'pxegrub',
            'option_code150_value' => 'text',
            'omapiport' => 1234,
            'pools' => {
              'internal' => {
                'network' => '10.0.0.0',
                'mask' => '255.255.255.0',
                'range' => ['10.0.0.180 10.0.0.190'],
                'gateway' => '10.0.0.1',
              },
            },
            'hosts' => {
              'mg5250' => {
                'mac' => '00:1E:8F:A2:B8:DD',
                'ip' => '10.0.0.99',
              },
            },
          },
        }
      end

      it {
        is_expected.to compile

        is_expected.to contain_class('dhcp')
          .with(
            'service_ensure'       => 'running',
            'interfaces'           => ['ens192'],
            'nameservers'          => ['127.0.0.1'],
            'ntpservers'           => ['127.0.0.1'],
            'pxeserver'            => '127.0.0.1',
            'dnsdomain'            => ['localdomain'],
            'pxefilename'          => 'pxelinux.0',
            'omapi_port'           => 1234,
            'ddns_update_style'    => 'omapiport',
            'max_lease_time'       => 86_400,
            'default_lease_time'   => 86_400,
            'logfacility'          => 'local7',
            'option_code150_label' => 'pxegrub',
            'option_code150_value' => 'text',
          )

        is_expected.to contain_dhcp__pool('internal')
          .with(
            'network' => '10.0.0.0',
            'mask' => '255.255.255.0',
            'range' => ['10.0.0.180 10.0.0.190'],
            'gateway' => '10.0.0.1',
          )

        is_expected.to contain_dhcp__host('mg5250')
          .with(
            'mac' => '00:1E:8F:A2:B8:DD',
            'ip' => '10.0.0.99',
          )
      }
    end
  end
end
