require 'spec_helper'

describe 'pxe_install::kickstart' do
  let(:title) { 'test02' }

  on_supported_os.each do |os, os_facts|
    context "on #{os}" do
      let(:facts) { os_facts }
      let(:pre_condition) do
        <<-EOF
        include ::pxe_install
        EOF
      end

      let(:params) do
        {
          'data' => {
            'ensure' => 'present',
            'network' => {
              'hostname' => 'test02',
              'domain' => ' localdomain',
              'mac' => '08:00:27:77:A4:41',
              'pxefile' => 'pxelinux.0',
              'prefix' => 'ubuntu-installer/amd64',
              'noprompt' => 'noprompt',
              'fixedaddress' => '10.0.0.67',
              'ksdevice' => 'eth0',
              'gateway' => '10.0.0.1',
              'netmask' => '255.255.255.0',
              'dns' => ['10.0.0.62'],
            },
            'rootpw' => '$1$XlmxMqOR$OvdmPRMDMpB6HCmmTYkY51',
            'timezone' => 'Europe/Berlin',
            'ostype' => 'ubuntu',
            'keyboard' => 'de(Macintosh, no dead keys)',
            'keymap' => 'de',
            'language' => 'de',
            'country' => 'DE',
            'locale' => 'en_US.UTF-8',
            'path' => 'ubuntu-installer/amd64/boot-screens',
            'loghost' => '10.0.0.2',
            'logport' => 514,
            'parameter' => {
              'env' => 'production',
              'role' => 'test',
              'dc' => 'test',
            },
            'partitioning' => {
              '/boot' => {
                'min' => 1024,
                'prio' => 1024,
                'max' => 1024,
                'fstype' => 'ext3',
                'primary' => true,
                'bootable' => true,
                'format' => true,
                'label' => 'boot',
                'method' => 'format',
                'filesystem' => 'ext3',
              },
            },
          },
          'kickstart_dir' => '/export/repos/kickstart',
          'kickstart_url' => 'http://localhost/kickstart',
          'repos_url' => 'http://localhost/repos',
          'scripturl' => 'http://localhost/pub',
        }
      end

      it {
        is_expected.to compile

        is_expected.to contain_concat('/export/repos/kickstart/test02')
          .with(
            'ensure' => 'present',
            'owner'  => 'root',
            'group'  => 'root',
            'mode'   => '0644',
          )

        is_expected.to contain_pxe_install__partitioning__ubuntu('test02')
          .with(
            'hostname'       => 'test02',
            'partitioning'   => {
              '/boot' => {
                'min' => 1024,
                'prio' => 1024,
                'max' => 1024,
                'fstype' => 'ext3',
                'primary' => true,
                'bootable' => true,
                'format' => true,
                'label' => 'boot',
                'method' => 'format',
                'filesystem' => 'ext3',
              },
            },
            'kickstart_file' => '/export/repos/kickstart/test02',
          )
      }
    end
  end
end
